/*
 * MPQCJob.hpp
 *
 *  Created on: Jul 10, 2012
 *      Author: heber
 */

#ifndef MPQCJOB_HPP_
#define MPQCJOB_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "boost/serialization/export.hpp"

#include "JobMarket/Jobs/FragmentJob.hpp"

#include "Jobs/Grid/SamplingGridProperties.hpp"

#include <string>

/** This class encapsulates a MPQC Job.
 *
 */
class MPQCJob : public FragmentJob
{
public:
  MPQCJob(
      const JobId_t _JobId,
      const std::string &_inputfile,
      const double _begin[3],
      const double _size,
      const int _level);
  virtual ~MPQCJob();

  FragmentResult::ptr Work();

private:
  //!> contents of inputfile
  const std::string inputfile;
  //!> information for how to sample the density
  const SamplingGridProperties grid;

private:
  /** private default cstor for serialization only
   */
  MPQCJob();

  friend class boost::serialization::access;
  // serialization
  template <typename Archive>
  void serialize(Archive& ar, const unsigned int version)
  {
    ar & boost::serialization::base_object<FragmentJob>(*this);
    ar & const_cast<std::string &>(inputfile);
    ar & const_cast<SamplingGridProperties &>(grid);
  }
};

// we need to give this class a unique key for serialization
// its is only serialized through its base class FragmentJob
BOOST_CLASS_EXPORT_KEY(MPQCJob)

#endif /* MPQCJOB_HPP_ */
